diff --git a/src/bin/scripts/.gitignore b/src/bin/scripts/.gitignore
new file mode 100644
index e62f4b0..2cc8f24
*** a/src/bin/scripts/.gitignore
--- b/src/bin/scripts/.gitignore
***************
*** 7,12 ****
--- 7,13 ----
  /dropuser
  /reindexdb
  /vacuumdb
+ /pg_ping
  
  /dumputils.c
  /keywords.c
diff --git a/src/bin/scripts/Makefile b/src/bin/scripts/Makefile
new file mode 100644
index 0980b4c..fc8a12b
*** a/src/bin/scripts/Makefile
--- b/src/bin/scripts/Makefile
*************** subdir = src/bin/scripts
*** 16,22 ****
  top_builddir = ../../..
  include $(top_builddir)/src/Makefile.global
  
! PROGRAMS = createdb createlang createuser dropdb droplang dropuser clusterdb vacuumdb reindexdb
  
  override CPPFLAGS := -I$(top_srcdir)/src/bin/pg_dump -I$(top_srcdir)/src/bin/psql -I$(libpq_srcdir) $(CPPFLAGS)
  
--- 16,22 ----
  top_builddir = ../../..
  include $(top_builddir)/src/Makefile.global
  
! PROGRAMS = createdb createlang createuser dropdb droplang dropuser clusterdb vacuumdb reindexdb pg_ping
  
  override CPPFLAGS := -I$(top_srcdir)/src/bin/pg_dump -I$(top_srcdir)/src/bin/psql -I$(libpq_srcdir) $(CPPFLAGS)
  
*************** dropuser: dropuser.o common.o dumputils.
*** 34,39 ****
--- 34,40 ----
  clusterdb: clusterdb.o common.o dumputils.o kwlookup.o keywords.o | submake-libpq
  vacuumdb: vacuumdb.o common.o | submake-libpq
  reindexdb: reindexdb.o common.o dumputils.o kwlookup.o keywords.o | submake-libpq
+ pg_ping: pg_ping.o common.o | submake-libpq submake-libpgport
  
  dumputils.c keywords.c: % : $(top_srcdir)/src/bin/pg_dump/%
  	rm -f $@ && $(LN_S) $< .
*************** install: all installdirs
*** 54,59 ****
--- 55,61 ----
  	$(INSTALL_PROGRAM) clusterdb$(X)  '$(DESTDIR)$(bindir)'/clusterdb$(X)
  	$(INSTALL_PROGRAM) vacuumdb$(X)   '$(DESTDIR)$(bindir)'/vacuumdb$(X)
  	$(INSTALL_PROGRAM) reindexdb$(X)  '$(DESTDIR)$(bindir)'/reindexdb$(X)
+ 	$(INSTALL_PROGRAM) pg_ping$(X)    '$(DESTDIR)$(bindir)'/pg_ping$(X)
  
  installdirs:
  	$(MKDIR_P) '$(DESTDIR)$(bindir)'
diff --git a/src/bin/scripts/pg_ping.c b/src/bin/scripts/pg_ping.c
new file mode 100644
index ...e4abc5a
*** a/src/bin/scripts/pg_ping.c
--- b/src/bin/scripts/pg_ping.c
***************
*** 0 ****
--- 1,184 ----
+ /*-------------------------------------------------------------------------
+  *
+  * pg_ping --- checks the status of the PostgreSQL server
+  *
+  * Copyright (c) 2012, PostgreSQL Global Development Group
+  *
+  * src/bin/scripts/pg_ping.c
+  *
+  *-------------------------------------------------------------------------
+  */
+ 
+ #include "postgres_fe.h"
+ #include "common.h"
+ 
+ void
+ help(const char *progname);
+ 
+ int
+ main(int argc, char **argv)
+ {
+     int c,optindex,opt_index = 0;
+ 
+     const char *progname = NULL;
+     const char *pghost = NULL;
+     const char *pgport = NULL;
+     const char *pguser = NULL; 
+     const char *pgdbname = NULL; 
+ 
+     const char *keywords[4], *values[4];
+ 
+     bool verbose = false;
+ 
+     PGPing rv;
+     PQconninfoOption *connect_options, *conn_opt_ptr;
+ 
+     static struct option long_options[] = {
+             {"dbname", required_argument, NULL, 'd'},
+             {"host", required_argument, NULL, 'h'},
+             {"port", required_argument, NULL, 'p'},
+             {"username", required_argument, NULL, 'U'},
+             {"version", no_argument, NULL, 'V'},
+             {"verbose", no_argument, NULL, 'v'},
+             {NULL, 0, NULL, 0}
+         };
+ 
+     progname = get_progname(argv[0]);
+ 
+     if ((argc > 1) && (strcmp(argv[1], "--help") == 0 || strcmp(argv[1], "-?") == 0))
+     {
+         help(progname);
+         exit(3); /* Return UNKNOWN */
+     }
+ 
+     while ((c = getopt_long(argc, argv, "d:h:p:U:vV", long_options, &optindex)) != -1)
+     {
+         switch (c)
+         {
+             case 'd':
+                 pgdbname = pg_strdup(optarg);
+                 break;
+             case 'h':
+                 pghost = pg_strdup(optarg);
+                 break;
+             case 'p':
+                 pgport = pg_strdup(optarg);
+                 break;
+             case 'U':
+                 pguser = pg_strdup(optarg);
+                 break;
+             case 'v':
+                 verbose = true;
+                 break;
+             case 'V':
+                 puts("pg_ping (PostgreSQL) " PG_VERSION);
+             default:
+                 exit(3); /* Return UNKNOWN*/
+         }
+     }
+ 
+     connect_options = PQconndefaults();
+     conn_opt_ptr = connect_options;
+ 
+     while (conn_opt_ptr->keyword)
+     {
+         if (strncmp(conn_opt_ptr->keyword, "host", 5) == 0)
+         {
+             if (pghost)
+             {
+                 keywords[opt_index] = conn_opt_ptr->keyword;
+                 values[opt_index] = pghost;
+                 opt_index++;
+             }
+             else if (conn_opt_ptr->val)
+                 pghost = conn_opt_ptr->val;
+             else
+                 pghost = DEFAULT_PGSOCKET_DIR;
+         }
+         else if (strncmp(conn_opt_ptr->keyword, "port", 5) == 0)
+         {
+             if (pgport)
+             {
+                 keywords[opt_index] = conn_opt_ptr->keyword;
+                 values[opt_index] = pgport;
+                 opt_index++;
+             }
+             else if (conn_opt_ptr->val)
+                 pgport = conn_opt_ptr->val;
+         }
+         else if (strncmp(conn_opt_ptr->keyword, "user", 5) == 0)
+         {
+             if (pguser)
+             {
+                 keywords[opt_index] = conn_opt_ptr->keyword;
+                 values[opt_index] = pguser;
+                 opt_index++;
+             }
+             else if (conn_opt_ptr->val)
+                 pguser = conn_opt_ptr->val;
+         }
+         else if (strncmp(conn_opt_ptr->keyword, "dbname", 7) == 0)
+         {
+             if (pgdbname)
+             {
+                 keywords[opt_index] = conn_opt_ptr->keyword;
+                 values[opt_index] = pgdbname;
+                 opt_index++;
+             }
+             else if (conn_opt_ptr->val)
+                 pgdbname = conn_opt_ptr->val;
+         }
+         conn_opt_ptr++;
+     }
+ 
+     keywords[opt_index] = NULL;
+     values[opt_index] = NULL;
+ 
+     rv = PQpingParams(keywords, values, 1);
+ 
+     if (verbose)
+     {
+         printf("%s:%s - ", pghost, pgport);
+ 
+         switch (rv)
+         {
+             case PQPING_OK:
+                 printf("Accepting Connections\n");
+                 break;
+             case PQPING_REJECT:
+                 printf("Rejecting Connections\n");
+                 break;
+             case PQPING_NO_RESPONSE:
+                 printf("No Response\n");
+                 break;
+             case PQPING_NO_ATTEMPT:
+                 printf("No Attempt\n");
+                 break;
+             default:
+                 printf("Unknown\n");
+         }
+     }
+ 
+     PQconninfoFree(connect_options);
+ 
+     exit(rv);
+ }
+ 
+ void
+ help(const char *progname)
+ {
+     printf(_("%s issues a connection check to a PostgreSQL database.\n\n"), progname);
+     printf(_("Usage:\n"));
+     printf(_("  %s [OPTION]...\n"), progname);
+ 
+     printf(_("\nOptions:\n"));
+     printf(_("  -d, --dbname=DBNAME      database name\n"));
+     printf(_("  -U, --username=USERNAME  database username\n"));
+     printf(_("  -V, --version            output version information, then exit\n"));
+     printf(_("  -v, --verbose            output verbose messages\n"));
+     printf(_("  -?, --help               show this help, then exit\n"));
+ 
+     printf(_("\nConnection options:\n"));
+     printf(_("  -h, --host=HOSTNAME      database server host or socket directory\n"));
+     printf(_("  -p, --port=PORT          database server port\n"));
+ }
